﻿using System;
using System.Collections;
using System.Collections.Generic;
using UnityEngine;
using UnityEngine.SceneManagement;

// 타일 정보
[Serializable]
public class CTileInfo
{
    public int width;   // 타일 가로 갯수
    public int height;  // 타일 세로 갯수
    public List<string> tiletypes; // 타일 타입 정보 배열
    public List<string> tiletypes1; // 타일 타입 정보 배열
    public List<string> tiletypes2; // 타일 타입 정보 배열
}

public class CTileManager : MonoBehaviour {

    public Transform _rootTile; // 최상위 타일 오브젝트

    public TextAsset _tileInfoJsonFile; // 타일 정보 파일

    public CTileInfo _tileInfo; // 타일 정보 객체

    public GameObject[] _tilePrefab; // 타일 프리팹

    public static int formula = 1; // 길찾기 방식

    public int[,] _tileTypes; // 타일 타입 정보 배열(2차원)

    public CPathNode[,] _tiles; // 경로 정보 2차원 배열

    // * tiles 정보 배열과 인덱스가 일치해야 함
    public GameObject[,] _tileObjects; // 타일 오브젝트 2차원 배열

    public float _tileSize = 0.32f; // 타일 블록 크기

    public GameObject[] winImage;

    public GameObject Popup;

    public int lv = 0;

    public bool popupOpenBool = false;

    void Awake()
    {
        CreateTiles();
    }

    void Update()
    {
        if (Input.GetKeyDown(KeyCode.Space) && popupOpenBool == true)
        {
            SceneManager.LoadScene("End");
        }
    }

    public void OpenPopUp()
    {
        Popup.SetActive(true);
        popupOpenBool = true;

    }

    public void XTiles() {
        

        Transform[] childList = _rootTile.GetComponentsInChildren<Transform>(true);
        if (childList != null)
        {
            for (int i = 0; i < childList.Length; i++)
            {
                if (childList[i] != transform)
                    Destroy(childList[i].gameObject);
            }
        }

    }

    public void CreateTiles()
    {
        XTiles();

        // 타일 정보 객체 생성
        _tileInfo = JsonUtility.FromJson<CTileInfo>(_tileInfoJsonFile.text);

        // 타입 정보 배열 객체 생성
        _tileTypes = new int[_tileInfo.width, _tileInfo.height];

        // 타일의 타입정보를 타일 정보 파일을 참조하여 타입 정보 배열을 생성함
        int y = 0;
        for (int i = _tileInfo.height - 1; i >= 0; i--)
        {
            // 정보 파일의 가장 배열 아래쪽 스트링 정보를 로드함

            
            string indexString;
            if (lv == 0)
            {
                indexString = _tileInfo.tiletypes[i];
            } else if (lv == 1)
            {
                indexString = _tileInfo.tiletypes1[i];
            }
            else
            {
                indexString = _tileInfo.tiletypes2[i];
            }

            // , 기호를 기준으로 타입 문자열을 배열로 추출함
            string[] strs = indexString.Split(",".ToCharArray(), System.StringSplitOptions.None);

            // 타일 타입 요소를 타일 타입 정보 배열에 위치에 맞게 저장함
            for (int x=0; x<_tileInfo.width; x++) // x축 순회
            {
                _tileTypes[x, y] = int.Parse(strs[x]);
            }
            y++; // y축 증가
        }

        // 타일 오브젝트 배열 생성
        _tileObjects = new GameObject[_tileInfo.width, _tileInfo.height];

        // 타일 정보 배열 생성
        _tiles = new CPathNode[_tileInfo.width, _tileInfo.height];

        // 배열의 타입 정보에 맞는 타일 오브젝트를 생성함
        for (y = 0; y < _tileInfo.height; y++)
        {
            for (int x = 0; x < _tileInfo.width; x++)
            {
                // 오브젝트 위치 계산
                Vector2 pos = new Vector2(x * _tileSize, y * _tileSize);

                int tileType = _tileTypes[x, y];

                if (tileType == 0)
                {
                    float r = UnityEngine.Random.Range(0, 100);
                    if (r > 75)
                    {
                        tileType = 1;
                    }
                }

                if (tileType == 3)
                {
                    tileType = 0;
                }
                if (tileType == 9)
                {
                    tileType = 2;
                }

                // 타일에 타입에 맞는 타일 오브젝트를 생성함
                _tileObjects[x, y] = Instantiate(_tilePrefab[tileType], pos, Quaternion.identity);

                // 타일 정보 배열에 타일 정보 객체를 생성해서 참조함
                _tiles[x, y] = new CPathNode();

                // 타일의 위치를 이름으로 설정함
                _tileObjects[x, y].name = x + ", " + y;

                _tiles[x, y].X = x; // 타일의 x 위치 설정
                _tiles[x, y].Y = y; // 타일의 y 위치 설정
                // 타일의 이동 가능 여부 정보에 대한 설정
                // true : 이동 불가, false : 이동 가능
                _tiles[x, y].IsWall = (tileType == 0) ? true : false;

                // 현재 타일 오브젝트를 Root 타일의 자식으로 설정함
                _tileObjects[x, y].transform.SetParent(_rootTile);

            }
        }
    }

    public void ReStart() {
        SceneManager.LoadScene("End");
    }
}
